///////////////////////////////////////////////////////////////////////////
//                           Source Arranger                             //
//                             Version 1.3                               //
//                Copyright (C) 1996,1997 Fumio Kawamata                 //
//                                                                       //
//  This library is free software; you can redistribute it and/or modify //
//  it under the terms of the GNU Library General Public License as      //
//  published by the Free Software Foundation; version 2 of the License. //
//                                                                       //
//  This library is distributed in the hope that it will be useful, but  //
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT- //
//  ABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library    //
//  General Public License in the directory "GNUGPL" for more details.   //
///////////////////////////////////////////////////////////////////////////
//*************************************************************************
//*
//*************************************************************************
unit Main;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Menus, Clipbrd, ExtCtrls, ComCtrls;

type
  TformMain = class(TForm)
    memoClip                    : TMemo;
    menuEdit                    : TMenuItem;
    menuEditCopy                : TMenuItem;
    menuEditPaste               : TMenuItem;
    menuEditSeparator1          : TMenuItem;
    menuFile                    : TMenuItem;
    menuFileBoth                : TMenuItem;
    menuFileExit                : TMenuItem;
    menuFileJustify             : TMenuItem;
    menuFileSort                : TMenuItem;
    menuHelp                    : TMenuItem;
    menuHelpAbout               : TMenuItem;
    menuMain                    : TMainMenu;
    menuOption                  : TMenuItem;
    menuOptionCaseSensitive     : TMenuItem;
    menuOptionCurrentMargin     : TMenuItem;
    menuOptionCurrentSeparator  : TMenuItem;
    menuOptionCurrentStatus     : TMenuItem;
    menuOptionCurrentTabWidth   : TMenuItem;
    menuOptionMargin            : TMenuItem;
    menuOptionSeparator         : TMenuItem;
    menuOptionTabWidth          : TMenuItem;
    menuOptionUseTab            : TMenuItem;
    statusbar1                  : TStatusBar;
    timerStatusDisplay          : TTimer;
    menuEditSelectAll: TMenuItem;
    menuEditCut: TMenuItem;
    procedure menuEditCopyClick(Sender: TObject);
    procedure menuEditCutClick(Sender: TObject);
    procedure menuEditPasteClick(Sender: TObject);
    procedure menuEditSelectAllClick(Sender: TObject);
    procedure menuFileBothClick(Sender: TObject);
    procedure menuFileExitClick(Sender: TObject);
    procedure menuFileJustifyClick(Sender: TObject);
    procedure menuFileSortClick(Sender: TObject);
    procedure menuHelpAboutClick(Sender: TObject);
    procedure menuOptionCaseSensitiveClick(Sender: TObject);
    procedure menuOptionMarginClick(Sender: TObject);
    procedure menuOptionSeparatorClick(Sender: TObject);
    procedure menuOptionTabWidthClick(Sender: TObject);
    procedure menuOptionUseTabClick(Sender: TObject);
    procedure onFormClose(Sender: TObject; var Action: TCloseAction);
    procedure onFormCreate(Sender: TObject);
    procedure timerStatusDisplayTimer(Sender: TObject);
    private
    procedure DisplayStatus(Mess: string);
    procedure SetStatuses;
    public
  end;


var
  formMain: TformMain;

implementation

uses MyFunc;

{$R *.DFM}

type
  DummyStrArray = array[1..1] of string[255];  // Fails without "[255]"
  StrArrayPointer = ^DummyStrArray;


var
  CaseSensitive     : boolean;
  Margin            : word;
  Separator         : string;
  TabWidth          : word;
  UseTab            : boolean;


//*************************************************************************
//* Justify string
//*************************************************************************
procedure JustifyStrArray(var StrArray:StrArrayPointer; N:integer);
var
    I               : word;
    L               : word;
    MarginStr       : string;
    MaxVarLength    : word;
    S               : string;
    SpaceStr        : string;
    VarName         : string;
    VarWidth        : word;
begin
  for I:=1 to N do begin
    S:=StrArray^[I];
    StrArray^[I]:=MyFunc.TabToSpace(S);
  end;
  MaxVarLength:=0;
  for I:=1 to N do begin
    S:=StrArray^[I];
    VarName:=MyFunc.ShiftWord(S,Separator);
    VarName:=MyFunc.Trim(VarName);
    L:=Length(VarName);
    if L>MaxVarLength then MaxVarLength:=L;
  end;
  VarWidth:=((Margin+MaxVarLength+TabWidth) div TabWidth)*TabWidth;
  MarginStr:=MyFunc.Spacing(Margin);
  for I:=1 to N do begin
    S:=StrArray^[I];
    VarName:=MyFunc.ShiftWord(S,Separator);
    VarName:=MyFunc.Trim(VarName);
    L:=length(VarName);
    SpaceStr:=MyFunc.Spacing(VarWidth-Margin-L);
    StrArray^[I]:=MarginStr+VarName+SpaceStr+Separator+' '+MyFunc.Trim(S);
  end;
end;


//*************************************************************************
//* Sorts string
//*************************************************************************
procedure SortStrArray(var StrArray:StrArrayPointer; N:integer);
var
  CompareResult : integer;
  I : integer;
  J : integer;
  TmpStr :string;
begin
  for I:=1 to N-1 do
    begin
      for J:=I+1 to N do
        begin
          if CaseSensitive=True
            then CompareResult:=CompareStr(StrArray^[I], StrArray^[J])
            else CompareResult:=CompareText(StrArray^[I], StrArray^[J]);
          if CompareResult>0 then
            begin
              TmpStr:=StrArray^[I];
              StrArray^[I]:=StrArray^[J];
              StrArray^[J]:=TmpStr;
            end;
        end;
    end;
end;


//*************************************************************************
//* Reads statuses
//*************************************************************************
procedure ReadStatuses;
var
  BoolStr   : string;
  FileName  : string;
  IniFile   : textfile;
begin
  FileName:=MyFunc.ExtractFileNameWithoutExt(paramstr(0))+'.INI';
  if not(FileExists(FileName)) then exit;
  AssignFile(IniFile, FileName);
  Reset(IniFile);
  readln(IniFile);
  readln(IniFile);
  readln(IniFile,BoolStr);
  if BoolStr='TRUE' then CaseSensitive:=True
  else CaseSensitive:=False;
  readln(IniFile,Margin);
  readln(IniFile,Separator);
  readln(IniFile,TabWidth);
  readln(IniFile,BoolStr);
  if BoolStr='TRUE' then UseTab:=True
  else UseTab:=False;
  CloseFile(IniFile);
end;


//*************************************************************************
//* Displays Status string
//*************************************************************************
procedure TformMain.DisplayStatus(Mess : String);
begin
  with formMain.timerStatusDisplay do
    begin
      if Enabled=True
        then Enabled:=False;
      Enabled:=True;
    end;
  formMain.statusbar1.Panels[0].text:=Mess;
  formMain.statusbar1.update;
end;


//*************************************************************************
//* Sets statuses
//*************************************************************************
procedure TformMain.SetStatuses;
var
    FileName    : string;
    IniFile     : textfile;
begin
  menuOptionCurrentMargin.Caption:='Margin: '+IntToStr(Margin);
  menuOptionCurrentSeparator.Caption:='Separator: "'+Separator+'"';
  menuOptionCurrentTabWidth.Caption:='Tab Width: '+IntToStr(TabWidth);
  menuOptionUseTab.Checked:=UseTab;
  menuOptionCaseSensitive.Checked:=CaseSensitive;
  FileName:=MyFunc.ExtractFileNameWithoutExt(paramstr(0))+'.INI';
  AssignFile(IniFile, FileName);
  Rewrite(IniFile);
  Writeln(IniFile,'# Source Arranger');
  Writeln(IniFile,'# Never edit this file.');
  Writeln(IniFile,CaseSensitive);
  Writeln(IniFile,Margin);
  Writeln(IniFile,Separator);
  Writeln(IniFile,TabWidth);
  Writeln(IniFile,UseTab);
  CloseFile(IniFile);
end;


//*************************************************************************
//* onClick [Help|About] event handler
//*************************************************************************
procedure TformMain.menuHelpAboutClick(Sender: TObject);
const
  CR = chr($0D);
var
  Msg  : string;
begin
  DisplayStatus('Displaying About Dialog');
  Msg:=    '         Source Arranger Version 1.3' + CR + CR;
  Msg:=Msg+'       Copyright(C) 1996, 1997 Fumio Kawamata' + CR + CR;
  Msg:=Msg+'This program is free software distributed under GNU GPL.' + CR;
  MessageDlg(Msg, mtInformation,[mbOK],0);
end;


//*************************************************************************
//* onClick [File|Both] event handler
//*************************************************************************
procedure TformMain.menuFileBothClick(Sender: TObject);
var
    I               : word;
    N       : integer;
    StrArray        : StrArrayPointer;
begin
  N:=memoClip.Lines.Count;
  if N=0 then begin
    MessageDlg('There is no text to justify & sort.', mtInformation,[mbOK],0);
    exit;
  end;
  GetMem(StrArray, N*256);
  for I:=0 to N-1 do StrArray^[I+1]:=memoClip.Lines[I];
  JustifyStrArray(StrArray, N);
  SortStrArray(StrArray, N);
  memoClip.Lines.Clear;
  for I:=1 to N do memoClip.Lines.Add(StrArray^[I]);
  FreeMem(StrArray, N*256);
  memoClip.SelectAll;
  DisplayStatus('Justifyed & Sorted');
end;


//*************************************************************************
//* onClick [Option|Case Sensitive] event handler
//*************************************************************************
procedure TformMain.menuOptionCaseSensitiveClick(Sender: TObject);
var
  StatusStr : string;
begin
  CaseSensitive:=not(CaseSensitive);
  if CaseSensitive=True then StatusStr:='True'
  else StatusStr:='False';
  SetStatuses;
  DisplayStatus('Current Case Sensitive status is '+StatusStr);
end;


//*************************************************************************
//* onClick [Edit|Copy] event handler
//*************************************************************************
procedure TformMain.menuEditCopyClick(Sender: TObject);
begin
  memoClip.CopyToClipboard;
  DisplayStatus('Copyed To Clipboard');
end;


//*************************************************************************
//* onClick [File|Exit] event handler
//*************************************************************************
procedure TformMain.menuFileExitClick(Sender: TObject);
begin
  Close
end;


//*************************************************************************
//* onClick [File|Justify] event handler
//*************************************************************************
procedure TformMain.menuFileJustifyClick(Sender: TObject);
var
    I               : word;
    N       : integer;
    StrArray        : StrArrayPointer;
begin
  N:=memoClip.Lines.Count;
  if N=0 then begin
    MessageDlg('There is no text to justify.', mtInformation,[mbOK],0);
    exit;
  end;
  GetMem(StrArray, N*256);
  for I:=0 to N-1 do StrArray^[I+1]:=memoClip.Lines[I];
  JustifyStrArray(StrArray, N);
  memoClip.Lines.Clear;
  for I:=1 to N do memoClip.Lines.Add(StrArray^[I]);
  FreeMem(StrArray, N*256);
  memoClip.SelectAll;
  DisplayStatus('Justifyed');
end;


//*************************************************************************
//* onClick [Edit|Paste] event handler
//*************************************************************************
procedure TformMain.menuEditPasteClick(Sender: TObject);
begin
  if Clipboard.HasFormat(CF_TEXT)
    then
      begin
        memoClip.PasteFromClipboard;
      end
    else
      MessageDlg('Clipboard is empty.', mtInformation,[mbOK],0);
end;


//*************************************************************************
//* onClick [File|Sort] event handler
//*************************************************************************
procedure TformMain.menuFileSortClick(Sender: TObject);
var
  I         : word;
  N         : integer;
  StrArray  : StrArrayPointer;
begin
  N:=memoClip.Lines.Count;
  if N=0 then begin
    MessageDlg('There is no text to sort.', mtInformation,[mbOK],0);
    exit;
  end;
  GetMem(StrArray, N*256);
  for I:=0 to N-1 do StrArray^[I+1]:=memoClip.Lines[I];
  SortStrArray(StrArray, N);
  memoClip.Lines.Clear;
  for I:=1 to N do memoClip.Lines.Add(StrArray^[I]);
  FreeMem(StrArray,N*256);
  memoClip.SelectAll;
  DisplayStatus('Sorted');
end;


//*************************************************************************
//* onFormClose event handler
//*************************************************************************
procedure TformMain.onFormClose(Sender: TObject; var Action: TCloseAction);
begin
  Application.Terminate;
end;


//*************************************************************************
//* onFormCreate event handler
//*************************************************************************
procedure TformMain.onFormCreate(Sender: TObject);
begin
  CaseSensitive:=False;
  Margin:=2;
  Separator:=':';
  TabWidth:=4;
  UseTab:=False;
  ReadStatuses;
  SetStatuses;
end;


//*************************************************************************
//* onClick [Option|Use Tab] event handler
//*************************************************************************
procedure TformMain.menuOptionUseTabClick(Sender: TObject);
var
  UseTabStr : string;
begin
  UseTab:=not(UseTab);
  if UseTab=True then UseTabStr:='True'
  else UseTabStr:='False';
  SetStatuses;
  DisplayStatus('Current Use Tab status is '+UseTabStr);
end;


//*************************************************************************
//* onClick [Option|Margin] event handler
//*************************************************************************
procedure TformMain.menuOptionMarginClick(Sender: TObject);
var
  Str      : string;
begin
  Str:=InputBox('Margin','Please enter left margin.',IntToStr(Margin));
  Margin:=StrToInt(Str);
  if Margin>255 then Margin:=255;
  SetStatuses;
  DisplayStatus('Current Margin is '+IntToStr(Margin));
end;


//*************************************************************************
//* onClick [Option|Separator] event handler
//*************************************************************************
procedure TformMain.menuOptionSeparatorClick(Sender: TObject);
begin
  Separator:=InputBox('Separator','Please enter separator.',
    Separator);
  SetStatuses;
  DisplayStatus('Current Separator is "'+Separator+'"');
end;


//*************************************************************************
//* onClick [Option|Tab Width] event handler
//*************************************************************************
procedure TformMain.menuOptionTabWidthClick(Sender: TObject);
var
  Str      : string;
begin
  Str:=InputBox('Tab Width','Please enter tab width.', IntToStr(TabWidth));
  TabWidth:=StrToInt(Str);
  if TabWidth>255 then TabWidth:=255;
  SetStatuses;
  DisplayStatus('Current Tab Width is '+IntToStr(TabWidth));
end;


//*************************************************************************
//* onTimer event handler
//*************************************************************************
procedure TformMain.timerStatusDisplayTimer(Sender: TObject);
begin
  timerStatusDisplay.Enabled:=False;
  formMain.statusbar1.Panels[0].text:='';
  formMain.statusbar1.update;
end;


//*************************************************************************
//* Cuts selected text in the memo component
//*************************************************************************
procedure TformMain.menuEditCutClick(Sender: TObject);
begin
  memoClip.CutToClipboard
end;


//*************************************************************************
//* Selects all text in the memo component
//*************************************************************************
procedure TformMain.menuEditSelectAllClick(Sender: TObject);
begin
  memoClip.SelectAll;
end;


end.

